@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 2492 2007-06-21 18:25:40Z richb $"
-__version__   = "$Revision: 2492 $"
-__date__      = "$Date: 2007-06-21 14:25:40 -0400 (Thu, 21 Jun 2007) $"
+__id__        = "$Id: braillegenerator.py 2708 2007-08-24 21:17:00Z eitani $"
+__version__   = "$Revision: 2708 $"
+__date__      = "$Date: 2007-08-24 17:17:00 -0400 (Fri, 24 Aug 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1086,6 +1086,21 @@ class BrailleGenerator:
 
         regions = []
 
+        # Check to see if this table cell contains an icon (image).
+        # If yes:
+        #   1/ Try to get a description for it and speak that.
+        #   2/ Treat the object of role type ROLE_IMAGE and speak
+        #      the role name.
+        # See bug #465989 for more details.
+        #
+        if obj.image:
+            if obj.image.imageDescription:
+                regions.append(obj.image.imageDescription)
+            obj.role = rolenames.ROLE_IMAGE
+            regions.extend(self._getBrailleRegionsForImage(obj))
+            obj.role = rolenames.ROLE_TABLE_CELL
+            return regions
+
         # If this table cell has 2 children and one of them has a
         # 'toggle' action and the other does not, then present this
         # as a checkbox where:
@@ -1134,6 +1149,21 @@ class BrailleGenerator:
                 if action.getName(i) == "toggle":
                     obj.role = rolenames.ROLE_CHECK_BOX
                     regions = self._getBrailleRegionsForCheckBox(obj)
+
+                    # If this table cell doesn't have any label associated 
+                    # with it then also braille the table column header.
+                    # See Orca bug #455230 for more details.
+                    #
+                    label = self._script.getDisplayedText( \
+                        self._script.getRealActiveDescendant(obj))
+                    action = obj.action
+                    if label == None or len(label) == 0:
+                        n = obj.parent.table.getColumnAtIndex(obj.index)
+                        header = obj.parent.table.getColumnHeader(n)
+                        accHeader = atspi.Accessible.makeAccessible(header)
+                        regions[0].append(braille.Region(" "))
+                        regions[0].append(braille.Region(accHeader.name))
+
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
@@ -1230,9 +1260,9 @@ class BrailleGenerator:
                         if i == column:
                             focusRowRegion = cellRegions[0]
                 regions = [rowRegions, focusRowRegion]
-                settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
             else:
                 regions = self._getBrailleRegionsForTableCell(obj)
+            settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
         else:
             regions = self._getBrailleRegionsForTableCell(obj)
 
